rm(list = ls())

library(MASS)
#install.packages("ManifoldOptim")
library(ManifoldOptim)
mod <- Module("ManifoldOptim_module", PACKAGE = "ManifoldOptim")
source("sg1D.R")
source("sgFG.R")

K <- 2   # 2 groups
r <- 100  # covariance matrix dimension
n_k <- rep(40, K)  # sample size for each group
d <- 100  # sample size for generate covariance matrix 
u <- 5  #rank and envelope dimension

set.seed(999)

# Model I data generation
Ok <- matrix(runif(u^2), u, u)
Ok <- qr.Q(qr(Ok))
O <- matrix(runif((r-u)^2), r-u, r-u) 
D <- diag(exp(seq(-2, 2, length.out=(r-u))))
Omega0 <- O %*% D %*% t(O)
Omegak <- array(0, c(u, u, K))
for(j in 1:K){
  Dk <- diag(j*(1:u))
  Omegak[, , j] <- Ok %*% Dk %*% t(Ok)
}

Gamma <- matrix(runif(r*u), r, u)
Gamma <- qr.Q(qr(Gamma))
Gamma0 <- qr.Q(qr(Gamma),complete=T)[,(u+1):r]

delta <- array(0, c(r, r, K))
for (j in 1:K) {
  delta[, , j] <- Gamma %*% Omegak[, , j] %*% t(Gamma) + Gamma0 %*% Omega0 %*% t(Gamma0)
  delta[, , j] <- delta[, , j]+0.001*diag(r)
}

D_cov <- array(0, c(r, r, sum(n_k)))
for (j in 1:K) {
  for (m in 1:n_k[j]) {  
    X <- mvrnorm(d, mu = rep(0, r), Sigma = delta[, , j])
    D_cov[, , m + sum(n_k[0:(j-1)])] <- (t(X) %*% X)/d
  }  
}

# define \barD_k, \barD
Dbar_k <- array(0, c(r, r, K))
n <- sum(n_k)
for (j in 1:K) {
  tmp3 <- D_cov[, , (sum(n_k[0:(j-1)]) + 1):sum(n_k[0:j])]
  Dbar_k[, , j] <- apply(tmp3, c(1, 2), sum)/n_k[j]
}
tmp5 <- (n_k/n)*Dbar_k[, , 1:K]
Dbar <- apply(tmp5, c(1, 2), sum)

# use 1D algorithm as initial value
X1_init <- gmanifold1D(Dbar, Dbar_k, n_k, d = u)
X1 <- gmanifoldFG(Dbar, Dbar_k, n_k, d=u, X1_init)

# use random generated basis as initial value
X2_init <- matrix(runif(r*u), r, u)
X2_init <- qr.Q(qr(X2_init))

X2 <- gmanifoldFG(Dbar, Dbar_k, n_k, d=u, X2_init)


X10 <- qr.Q(qr(X1),complete=T)[, (u+1):r]
X20 <- qr.Q(qr(X2),complete=T)[, (u+1):r]
dlt_k_init <- dlt_k_rdm <- array(0, c(r, r, K))
for (k in 1:K) {
  dlt_k_init[, , k] <- X1 %*% t(X1) %*% Dbar_k[ , , k] %*% X1 %*% t(X1) +
    X10 %*% t(X10) %*% Dbar %*% X10 %*% t(X10)
  dlt_k_rdm[, , k] <- X2 %*% t(X2) %*% Dbar_k[ , , k] %*% X2 %*% t(X2) +
    X20 %*% t(X20) %*% Dbar %*% X20 %*% t(X20)
}

# estimation error of CRS estimator
(mu_env_init <- norm(((delta[, , 1] - delta[, , 2])-(dlt_k_init[, , 1]-dlt_k_init[, , 2])), type = "F"))
(mu_env_rdm  <- norm(((delta[, , 1] - delta[, , 2])-(dlt_k_rdm[, , 1]-dlt_k_rdm[, , 2])), type = "F"))

# estimation error of sample estimator
(mu_smp <- norm(((delta[, , 1] - delta[, , 2])-(Dbar_k[, , 1]-Dbar_k[, , 2])), type = "F"))

# subspace estimation error
(mu_init <- norm((X1 %*% t(X1)-Gamma %*% t(Gamma)), type="F"))
(mu_rdm <- norm((X2 %*% t(X2)-Gamma %*% t(Gamma)), type="F"))





