##################################################
#    get initial value for 1D algorithm          #
##################################################
get_ini_gmanifold1D <- function(Sx, Sk, nk) {
  p <- dim(Sx)[2]
  K <- length(nk)
  n <- sum(nk)
  v <- eigen(Sx)$vectors
  
  for (k in 1:K) {
    v2 <- eigen(Sk[ , , k])$vectors
    v <- cbind(v, v2)
  }
  
  W0 <- v[, 1]
  FW0 <- n*log(det(t(W0) %*% solve(Sx) %*% W0))
  
  for (k in 1:K){
    FW0 <- FW0 + nk[k]*log(det(t(W0) %*% Sk[ , , k] %*% W0))
  }
  
  for (i in 2:((K+1)*p)) {
    W <- v[, i]
    FW <- n*log(det(t(W) %*% solve(Sx) %*% W))
    for (k in 1:K) {
      if (det(t(W) %*% Sk[ , , k] %*% W) > 0){
        FW <- FW + nk[k]*log(det(t(W) %*% Sk[ , , k] %*% W))
      }
    }
    if (FW < FW0) {
      W0 <- W
      FW0 <- FW
    }
  }
  return(W0)
}


##################################################
#         1D optimization solve for gamma        #
##################################################
first1D <- function(Sx, Sk, nk) {
  W0 <- get_ini_gmanifold1D(Sx, Sk, nk)
  n <- sum(nk)
  p <- dim(Sx)[2]
  K <- length(nk)
  mw <- function(w) { matrix(w, p, 1) }
  f <- function(w) {W <- mw(w); 
  f1 <- n*log(t(W) %*% solve(Sx) %*% W) 
  for(k in 1:K) {
    f1 <- f1 + nk[k]*log(det(t(W) %*% Sk[, , k] %*% W))
  }
  return(f1)                           
  }
  g <- function(w) {W <- mw(w); 
  g1 <- n*solve(Sx) %*% W %*% solve(t(W) %*% solve(Sx) %*% W) 
  for(k in 1:K) {
    g1 <- g1 + nk[k]*Sk[, , k] %*% W %*% solve(t(W) %*% Sk[, , k] %*% W)
  }
  g1 <-2*g1
  return(g1)    
  }
 
  prob <- new(mod$RProblem, f, g)
  mani.defn <- get.stiefel.defn(p, 1)
  mani.params <- get.manifold.params(IsCheckParams = FALSE)
  solver.params <- get.solver.params(Max_Iteration = 500, IsCheckParams = FALSE)
  gamma <- manifold.optim(prob, mani.defn, method = "RCG", mani.params = mani.params,
                          solver.params = solver.params, x0 = W0)
  return(matrix(gamma$xopt,p,1))
}

##################################################
#  1D optimization solve for envelope basis      #
################################################## 
gmanifold1D <- function(Sx, Sk, nk, d) {
  # 1D estimation of basis
  # Input: 
  #   sample covariance of X: Sx, p*p postive definite
  #   conditional sample covariances of X in each group: Sk, p*p*K array of 
  #     K matrices each has dimension p*p, postive definite
  #   groupsize: nk, K*1 vector of sample sizes in each group
  #   envelope dimension: d, integer between 1 and p
  # Output:
  #   the basis matrix: Ghat, semi-orthogonal p*u
  p <- dim(Sx)[2]
  K <- length(nk)
  
  if(d < p){
    Sxnew <- Sx
    Sknew <- Sk
    G <- matrix(0, p, d)
    G0 <- diag(p)
    for (k in 1:d) {
      gk <- first1D(Sxnew, Sknew, nk)
      G[, k] <- G0 %*% gk
      G0 <- qr.Q(qr(G[, 1:k]),complete=T)[,(k+1):p]
      Sxnew <- t(G0) %*% Sx %*% G0
      Sknew <- array(0, c((dim(Sknew)[1]-1), (dim(Sknew)[2]-1), K))
      for (j in 1:K) {
        Sknew[ , ,j] <- t(G0) %*% Sk[ , , j] %*% G0
      }
    }
    Ghat <- G
  } else { Ghat <- diag(p) }
  return(Ghat)
}
