
##################################################
#  FG optimization solve for envelope basis      #
##################################################
gmanifoldFG <- function(Sx, Sk, nk, d, G_ini) {
  # Estimating basis
  # Input: 
  #   sample covariance of X: Sx, p*p postive definite
  #   conditional sample covariances of X in each group: Sk, p*p*K array of 
  #       K matrices each has dimension p*p, postive definite
  #   groupsize: nk, K*1 vector of sample sizes in each group
  #   envelope dimension: d, integer between 1 and p
  #   initial guess of the basis matrix: Ghat, semi-orthogonal p*u
  # Output:
  #   Basis matrix: Ghat, semi-orthogonal p*u
  
  p <- dim(Sx)[2]
  n <- sum(nk)
  K <- length(nk)
  if (d < p) {
    mw <- function(w) { matrix(w, p, d) }
    f <- function(w) {W <- mw(w); 
    f1 <- n*log(det(t(W) %*% solve(Sx) %*% W))
    for(k in 1:K) {
      f1 <- f1 + nk[k]*log(det(t(W) %*% Sk[, , k] %*% W))
    }
    return(f1)                           
    }
    g <- function(w) {W <- mw(w); 
    g1 <- n*solve(Sx) %*% W %*% solve(t(W) %*% solve(Sx) %*% W) 
    for(k in 1:K) {
      g1 <- g1 + nk[k]*Sk[, , k] %*% W %*% solve(t(W) %*% Sk[, , k] %*% W)
    }
    g1 <-2*g1
    return(g1)    
    }
 
    prob <- new(mod$RProblem, f, g)
    mani.defn <- get.stiefel.defn(p, d)
    mani.params <- get.manifold.params(IsCheckParams = FALSE)
    solver.params <- get.solver.params(Max_Iteration = 500, IsCheckParams = FALSE)
    gamma <- manifold.optim(prob, mani.defn, method = "RCG", mani.params = mani.params,
                            solver.params = solver.params, x0 = G_ini)
    G_hat <- matrix(gamma$xopt,p,d)
  }
  else { G_hat <- diag(p)}
  return(G_hat)
}